#pragma once

#ifndef __GPSCONNECTION_H__
#define __GPSCONNECTION_H__

#include "resource.h"       // main symbols
#include "RMCSentence.h"
#include "GGASentence.h"
#include "SerialPort.h"
#include "GPSCom.h"
#include "GPSComCP.h"

/////////////////////////////////////////////////////////////////////////////
// CGPS
class ATL_NO_VTABLE CGPSConnection : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CGPSConnection, &CLSID_GPSConnection>,
	public ISupportErrorInfoImpl<&IID_IGPSConnection>,
	public IConnectionPointContainerImpl<CGPSConnection>,
	public IDispatchImpl<IGPSConnection, &IID_IGPSConnection, &LIBID_GPSCOMLib>,
  public IProvideClassInfo2Impl<&CLSID_GPSConnection, &__uuidof(_IGPSConnectionEvents), &LIBID_GPSCOMLib>,
  public CProxy_IGPSConnectionEvents<CGPSConnection>
{
public:
//Constructors / Destructors
	CGPSConnection();
	virtual ~CGPSConnection();

DECLARE_PROTECT_FINAL_CONSTRUCT()
DECLARE_NO_REGISTRY()

BEGIN_CONNECTION_POINT_MAP(CGPSConnection)
CONNECTION_POINT_ENTRY(DIID__IGPSConnectionEvents)
END_CONNECTION_POINT_MAP()

BEGIN_COM_MAP(CGPSConnection)
	COM_INTERFACE_ENTRY(IGPSConnection)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
  COM_INTERFACE_ENTRY(IProvideClassInfo)
  COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

  virtual HRESULT FinalConstruct();
  virtual void FinalRelease();

//Non COM related functions
  HRESULT Open(long nPort, long nDataBits, long nBaudRate, GPS_STOP_BITS StopBits, GPS_PARITY Parity, const CComBSTR& bstrInitString);
  void Close();
  static UINT _MonitorThread(LPVOID lpParameter);
  UINT MonitorThread();

//IGPSConnection
	STDMETHOD(get_DataBits)(long* pVal);
	STDMETHOD(get_BaudRate)(long* pVal);
	STDMETHOD(get_StopBits)(GPS_STOP_BITS* pVal);
	STDMETHOD(get_Parity)(GPS_PARITY* pVal);
	STDMETHOD(get_Port)(long* pVal);
  STDMETHOD(get_RMC)(IRMCSentence** ppVal);
  STDMETHOD(get_GGA)(IGGASentence** ppVal);
	STDMETHOD(get_InitString)(BSTR* pVal);

//Member variables
  long                      m_nDataBits;
  long                      m_nBaudRate;
  GPS_STOP_BITS             m_StopBits;
  GPS_PARITY                m_Parity;
  long                      m_nPort;
  CSerialPort               m_Port;
  CWinThread*               m_pMonitorThread;
  volatile LONG             m_lWantThreadShutdown;
  CComObject<CRMCSentence>* m_pRMCSentence;
  CComObject<CGGASentence>* m_pGGASentence;
  CComBSTR                  m_bstrInitString;
};

#endif //__GPSCONNECTION_H__
