////////////////////////// Includes /////////////////////////////////////////

#include "stdafx.h"
#include "GPSCom.h"
#include "GPSConnections.h"
#include "GPSConnection.h"


/////////////////////////// Macros / Defines ////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

typedef CComEnum < IEnumVARIANT, &IID_IEnumVARIANT, VARIANT, _Copy<VARIANT> > CComEnumVariant;


//////////////////////////// Implementation /////////////////////////////////

CGPSConnections::CGPSConnections()
{
}

STDMETHODIMP CGPSConnections::get_Count(long* pVal)
{
  //Validate our parameters
  if (pVal == NULL)
    return E_POINTER;

  //Synchronize access to the connections array
  CMyApp* pApp = GetApp();
  CComCritSecLock<CComAutoCriticalSection> lock(pApp->m_csConnections);
  *pVal = pApp->m_Connections.GetSize();

	return S_OK;
}

STDMETHODIMP CGPSConnections::Item(long nIndex, IGPSConnection** ppConnection)
{
  //Validate our parameters
  if (ppConnection == NULL)
    return E_POINTER;

  //Initialize out value to NULL
  *ppConnection = NULL;

  //Synchronize access to the connections array
  CMyApp* pApp = GetApp();
  CComCritSecLock<CComAutoCriticalSection> lock(pApp->m_csConnections);

  //Get the specified connection
  if (nIndex >= 0 && nIndex < pApp->m_Connections.GetSize())
  {
    CComObject<CGPSConnection>* pConnection = static_cast<CComObject<CGPSConnection>*>(pApp->m_Connections[nIndex]);
    ATLASSERT(pConnection);

    //Query for the IGPSConnection interface for this array element
    return pConnection->QueryInterface(IID_IGPSConnection, reinterpret_cast<void**>(ppConnection));
  }
  else
  {
    //Create the rich error info
    Error(IDS_INVALID_CONNECTIONS_INDEX, IID_IGPSConnections);

    return E_INVALIDARG;
  }

	return S_OK;
}

STDMETHODIMP CGPSConnections::get__NewEnum(IUnknown** ppVal)
{
  //Validate our parameters
  if (ppVal == NULL)
    return E_POINTER;

  //Set the Enumerator to NULL
  *ppVal = NULL;

  CComObject<CComEnumVariant>* pEnumerator = NULL;
  HRESULT hr = CComObject<CComEnumVariant>::CreateInstance(&pEnumerator);
  if (SUCCEEDED(hr))
  {
    //Synchronize access to the connections array
    CMyApp* pApp = GetApp();
    CComCritSecLock<CComAutoCriticalSection> lock(pApp->m_csConnections);
  
    //Allocate a VARIANT array which will be taken over by the enumerator
    int nConnections = pApp->m_Connections.GetSize();
    try
    {
      VARIANT* pVarData = NULL;
      pVarData = new VARIANT[nConnections];
      if (pVarData)
      {
        //Copy each item into the VARIANT arry
        for (long i=0; i<nConnections; i++)
        {
          //Initialize each variant
          VariantInit(&pVarData[i]);

          //Fill in each variant's IDispatch pointer
          pVarData[i].vt = VT_DISPATCH;
          pVarData[i].pdispVal = NULL;
          CComPtr<IGPSConnection> pConnection;
          hr = Item(i, &pConnection); 
          if (SUCCEEDED(hr))
            hr = pConnection->QueryInterface(IID_IDispatch, reinterpret_cast<void**>(&pVarData[i].pdispVal));
        } 

        //Initialize the enumerator
        hr = pEnumerator->Init(&pVarData[0], &pVarData[nConnections], 0, AtlFlagTakeOwnership);
        if (SUCCEEDED(hr))
          hr = pEnumerator->QueryInterface(IID_IUnknown, reinterpret_cast<void**>(ppVal));
        else
          delete pEnumerator;
      }
      else
      {
        hr = E_OUTOFMEMORY;
        delete pEnumerator;
      }
    }
    catch(CMemoryException* pEx)
    {
      hr = E_OUTOFMEMORY;
      pEx->Delete();
      delete pEnumerator;
    }
  }

	return hr;
}
