/////////////////// Includes ////////////////////////////////////////////

#include "stdafx.h"
#include "GPSCom.h"
#include "GPSPort.h"
#include "GPSConnection.h"


///////////////////////////////// Macros / Defines /////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


////////////////// Implementation //////////////////////////////////////

CGPSPort::CGPSPort() : m_nDataBits(8), //Initialize the RS232 settings to default values for NMEA 0183 
                       m_nBaudRate(4800),
                       m_StopBits(GPS_1_STOP_BIT),
                       m_Parity(GPS_PARITY_NONE)
{
}

STDMETHODIMP CGPSPort::get_Parity(GPS_PARITY* pVal)
{
  //validate our parameters
	if (pVal == NULL)
    return E_POINTER;

  *pVal = m_Parity;

	return S_OK;
}

STDMETHODIMP CGPSPort::put_Parity(GPS_PARITY newVal)
{
	//validate our parameters
  if (newVal < GPS_PARITY_NONE || newVal > GPS_PARITY_EVEN)
  {
    Error(IDS_INVALID_PARITY_VALUE, IID_IGPSPort);
    return E_INVALIDARG;
  }
  
  m_Parity = newVal;

	return S_OK;
}

STDMETHODIMP CGPSPort::get_StopBits(GPS_STOP_BITS* pVal)
{
	//validate our parameters
  if (pVal == NULL)
    return E_POINTER;

  *pVal = m_StopBits;

	return S_OK;
}

STDMETHODIMP CGPSPort::put_StopBits(GPS_STOP_BITS newVal)
{
	//validate our parameters
  if (newVal < GPS_1_STOP_BIT || newVal > GPS_2_STOP_BITS)
  {
    Error(IDS_INVALID_STOP_BITS_VALUE, IID_IGPSPort);
    return E_INVALIDARG;
  }
  
  m_StopBits = newVal;

	return S_OK;
}

STDMETHODIMP CGPSPort::get_BaudRate(long* pVal)
{
  //validate our parameters
  if (pVal == NULL)
    return E_POINTER;

  *pVal = m_nBaudRate;

	return S_OK;
}

STDMETHODIMP CGPSPort::put_BaudRate(long newVal)
{
  m_nBaudRate = newVal;

	return S_OK;
}

STDMETHODIMP CGPSPort::get_DataBits(long* pVal)
{
  //validate our parameters
	if (pVal == NULL)
    return E_POINTER;

  *pVal = m_nDataBits;

	return S_OK;
}

STDMETHODIMP CGPSPort::put_DataBits(long newVal)
{
  m_nDataBits = newVal;

	return S_OK;
}

STDMETHODIMP CGPSPort::CreateConnection(long Port, VARIANT_BOOL MakeNewSharable, IGPSConnection** ppGPSConnection)
{
  //Validate our parameters
  if (ppGPSConnection == NULL)
    return E_POINTER;

  //Create a COM CGPSConnection object using an ATL creator
  CComObject<CGPSConnection>* pConnection = NULL;
  HRESULT hr = CComObject<CGPSConnection>::CreateInstance(&pConnection);
  if (SUCCEEDED(hr))
  {
    //Open up the connection
    hr = pConnection->Open(Port, m_nDataBits, m_nBaudRate, m_StopBits, m_Parity, m_bstrInitString);
    if (FAILED(hr))
      return hr;

    //Query for the specific early bound interface which we want to hand back
    hr = pConnection->QueryInterface(IID_IGPSConnection, reinterpret_cast<void**>(ppGPSConnection));

    //add to module array of shared instances
    if (SUCCEEDED(hr) && (MakeNewSharable == VARIANT_TRUE))
    {
      //Synchronize access to the connections array
      CMyApp* pApp = GetApp();
      CComCritSecLock<CComAutoCriticalSection> lock(pApp->m_csConnections);
      pApp->m_Connections.Add(pConnection);
    }
  }

	return hr;
}

STDMETHODIMP CGPSPort::get_InitString(BSTR* pVal)
{
  //Validate out parameters
  if (pVal == NULL)
    return E_POINTER;

  return m_bstrInitString.CopyTo(pVal);
}

STDMETHODIMP CGPSPort::put_InitString(BSTR newVal)
{
  return m_bstrInitString.AssignBSTR(newVal);
}
