///////////////////////////////// Includes ////////////////////////////////////

#include "stdafx.h"
#include "NMEA.h"


///////////////////////////////// Macros / Defines /////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


//////////////////////////////// Implementation ///////////////////////////////

NMEA_BOOLEAN CNmeaSentence::Boolean(int nField) const
{
  CString sField(Field(nField));

  if (sField == _T("A"))
    return NMEA_True;
  else if (sField == _T("V"))
    return NMEA_False;
  else
    return NMEA_Unknown;
}

int CNmeaSentence::HexDigitToInt(char c)
{
  //What will be the return value from this function
  int rVal = 0;

  if ((c >= 'a') && (c <= 'f'))
    rVal =  c - 'a' + 10;
  else if ((c >= 'A') && (c <= 'F'))
    rVal =  c - 'A' + 10;
  else if ((c >= '0') && (c <= '9'))
    rVal = c - '0';

  return rVal;
}

DWORD CNmeaSentence::HexValue(const CString& sHexString)
{
  int nLen = sHexString.GetLength();
  DWORD rVal = 0;
  int nMultiplier = 1;
  for (int i=nLen-1; i>=0; i--)
  {
    TCHAR c = sHexString.GetAt(i);
    rVal += (HexDigitToInt(static_cast<char>(c)) * nMultiplier);
    nMultiplier *= 16;
  }

  return rVal;
}

BYTE CNmeaSentence::ComputeChecksum() const
{
  //What will be the return value from this function
  BYTE checksum = 0;

  int nLength = m_Sentence.GetLength();
  int i = 1; // Skip over the $ at the begining of the CNmeaSentence
  while (i < nLength && m_Sentence[i] != _T('*') && m_Sentence[i] != _T('\r') && m_Sentence[i] != _T('\n'))
  {
    checksum ^= static_cast<BYTE>(m_Sentence[i]);
    i++;
  }

  return checksum;
}

double CNmeaSentence::Double(int nField) const
{
  CString sField(Field(nField));
  return _tstof(sField);
}

NMEA_EASTWEST CNmeaSentence::EastOrWest(int nField) const
{
  CString sField(Field(nField));
  if (sField == _T("E"))
    return NMEA_East;
  else if (sField == _T("W"))
    return NMEA_West;
  else
    return NMEA_EW_Unknown;
}

NMEA_LONGITUDE CNmeaSentence::Longitude(int nPositionFieldNumber, int nEastingFieldNumber) const
{
  NMEA_LONGITUDE rVal;
  rVal.Value = Angle(nPositionFieldNumber);
  rVal.Easting = EastOrWest(nEastingFieldNumber);
  return rVal;
}

NMEA_LATITUDE CNmeaSentence::Latitude(int nPositionFieldNumber, int nNorthingFieldNumber) const
{
  NMEA_LATITUDE rVal;
  rVal.Value = Angle(nPositionFieldNumber);
  rVal.Northing = NorthOrSouth(nNorthingFieldNumber);
  return rVal;
}

CString CNmeaSentence::Field(int nDesiredField) const
{
  //What will be the return value from this function
  CString rVal;

  int i = 1;
  int nCurrentField = 0;
  int nLen = m_Sentence.GetLength();
  while (nCurrentField < nDesiredField && i < nLen)
  {
    if (m_Sentence[i] == _T(',') || m_Sentence[i] == _T('*'))
      nCurrentField++;
    i++;
  }

  if (nCurrentField == nDesiredField)
  {
    while (i < nLen && m_Sentence[i] != _T(',') && m_Sentence[i] != _T('*') && m_Sentence[i] != _T('\n') && m_Sentence[i] != _T('\r'))
    {
      rVal += m_Sentence[i];
      i++;
    }
  }

  return rVal;
}

int CNmeaSentence::Integer(int nField) const
{
  CString sInteger(Field(nField));
  return _ttoi(sInteger);
}

NMEA_BOOLEAN CNmeaSentence::IsChecksumBad(int nField) const
{
  //Checksums are optional, return TRUE if an existing checksum is known to be bad
  CString sChecksum(Field(nField));

  if (sChecksum.IsEmpty())
    return NMEA_Unknown;

  BYTE CalculatedCheckSum = ComputeChecksum();
  DWORD ReceivedCheckSum = HexValue(sChecksum);
  return (CalculatedCheckSum != ReceivedCheckSum) ? NMEA_True : NMEA_False;
}

NMEA_NORTHSOUTH CNmeaSentence::NorthOrSouth(int nField) const
{
  CString sField(Field(nField));
  if (sField == _T("N"))
    return NMEA_North;
  else if (sField == _T("S"))
    return NMEA_South;
  else
    return NMEA_NS_Unknown;
}

NMEA_TIME CNmeaSentence::Time(int nField) const
{
  NMEA_TIME rVal;
  memset(&rVal, 0, sizeof(rVal));

  CString sTemp(Field(nField));
  if (sTemp.GetLength() >= 6)
  {
    TCHAR sNumber[3];
    sNumber[0] = sTemp[0];
    sNumber[1] = sTemp[1];
    sNumber[2] = _T('\0');
    rVal.wHour = static_cast<WORD>(::_ttoi(sNumber));

    sNumber[0] = sTemp[2];
    sNumber[1] = sTemp[3];
    rVal.wMinute = static_cast<WORD>(::_ttoi(sNumber));

    sNumber[0] = sTemp[4];
    sNumber[1] = sTemp[5];
    rVal.wSecond = static_cast<WORD>(::_ttoi(sNumber));
  }

  return rVal;
}

NMEA_DATE CNmeaSentence::Date(int nField) const
{
  NMEA_DATE rVal;
  memset(&rVal, 0, sizeof(rVal));

  CString sTemp(Field(nField));
  if (sTemp.GetLength() >= 6)
  {
    TCHAR sNumber[3];

    sNumber[0] = sTemp[0];
    sNumber[1] = sTemp[1];
    sNumber[2] = _T('\0');
    rVal.wDay = static_cast<WORD>(::_ttoi(sNumber));

    sNumber[0] = sTemp[2];
    sNumber[1] = sTemp[3];
    rVal.wMonth = static_cast<WORD>(::_ttoi(sNumber));

    sNumber[0] = sTemp[4];
    sNumber[1] = sTemp[5];
    rVal.wYear = static_cast<WORD>(::_ttoi(sNumber));

    //intelligently adjust the year
    if (rVal.wYear < 80)
      rVal.wYear += 2000;
    else if (rVal.wYear >= 80)
      rVal.wYear += 1900;
  }

  return rVal;
}

NMEA_ANGLE CNmeaSentence::Angle(int nField) const
{
  NMEA_ANGLE rVal;
  memset(&rVal, 0, sizeof(rVal));

  CString sTemp(Field(nField));

  int nPointIndex = sTemp.Find(_T('.'));
  if (nPointIndex != -1)
  {
    CString sUnits(sTemp.Left(nPointIndex));
    int nValue = _ttoi(sUnits);
    rVal.Degrees = static_cast<WORD>(nValue/100);
    rVal.Minutes = static_cast<WORD>(nValue - rVal.Degrees*100);

    CString sFraction(sTemp.Right(sTemp.GetLength() - nPointIndex));
    double fValue = _tstof(sFraction);

    rVal.Seconds = static_cast<WORD>(fValue*60);
    rVal.CentiSeconds = static_cast<WORD>((fValue*60 - rVal.Seconds)*100.0);
  }

  return rVal;
}

DWORD CNmeaSentence::NmeaAngleToHundrethsOfSeconds(const NMEA_ANGLE& angle)
{
  return (angle.Degrees*360000) + (angle.Minutes*6000) + (angle.Seconds*100) + angle.CentiSeconds;
}

DWORD CNmeaSentence::NmeaBearingToHundrethsOfDegrees(const double& value)
{
  int nDigits = static_cast<int>(value);
  int nHundreths = static_cast<int>((value - nDigits)*100);
  return nDigits*100 + nHundreths;
}


CRMCResponse::CRMCResponse()
{
  Empty();
}

void CRMCResponse::Empty()
{
  memset(&m_Time, 0, sizeof(m_Time));
  memset(&m_Date, 0, sizeof(m_Date));
  m_DataValid = NMEA_Unknown;
  memset(&m_Latitude, 0, sizeof(m_Latitude));
  memset(&m_Longitude, 0, sizeof(m_Longitude));
  m_SpeedOverGroundKnots = 0.0;
  m_Bearing = 0.0;
}

BOOL CRMCResponse::Parse(const CNmeaSentence& sentence)
{
  //First we check the checksum
  NMEA_BOOLEAN check = sentence.IsChecksumBad(12);
  if (check == NMEA_True)
    return FALSE;
  else if (check == NMEA_Unknown)
    return FALSE;

  //then the data
  m_Time                 = sentence.Time(1);
  m_Date                 = sentence.Date(9);
  m_DataValid            = sentence.Boolean(2);
  m_Latitude             = sentence.Latitude(3, 4);
  m_Longitude            = sentence.Longitude(5, 6);
  m_SpeedOverGroundKnots = sentence.Double(7);
  m_Bearing              = sentence.Double(8);

  return TRUE;
}


CGGAResponse::CGGAResponse()
{
  Empty();
}

void CGGAResponse::Empty()
{
  m_Fix                   = NMEA_FIX_NOT_AVAILABLE;
  m_nSatellites           = 0;
  m_AntennaAltitudeMeters = 0.0;
}

BOOL CGGAResponse::Parse(const CNmeaSentence& sentence)
{
  //First we check the checksum
  if (sentence.IsChecksumBad(15) == NMEA_True)
    return FALSE;

  //Then the data
  m_Fix                   = static_cast<NMEA_FIX>(sentence.Integer(6));
  m_nSatellites           = sentence.Integer(7);
  m_AntennaAltitudeMeters = sentence.Double(9);

  return TRUE;
}
